
	program read

!=====================================================================
!  DECLARATIONS
!=====================================================================

	implicit none

! Include files needed for ADAMO and DAD use

#include "partap.inc"
#include "dad/command.inc"

! Window common block (WCB) definitions for all tables you want to use

#include "mcEvKey.inc"
#include "mcEvent.inc"
#include "mcTrack.inc"

! Finally declare some variables of your own

	integer		maxtracks, ntracks
	parameter	(maxtracks=4)
	logical		use_key
	integer		i, iopt, igaf, idfl, iok, ievent, dadflags
	real		momentum(maxtracks)
	character	ckey*80, cline*80, cfile*80, cdriver*4

! ... external functions are declared just like variables
	integer		lnblnk		! useful CERNLIB routine


!=====================================================================
!  ASK USER TO SELECT OPTION: TYPE OF FILE TO READ
!=====================================================================

	write(6,'(/a)') 'Please select type of file to read: [<CR>=0]'
	write(6,901) 0, 'GAF in IE format'
	write(6,901) 1, 'GAF in FZ format'
	write(6,901) 2, 'DAD file (WFIL) via dadOpenFile'
	write(6,901) 3, 'DAD file (WFIL) via dadDoConnect'
	write(6,901) 4, 'DAD file (WFIL) via dadHost, dadConnect'
	write(6,901) 5, 'DAD file (WFIL) via dadInit, dadConnect'
	write(6,901) 6, 'DAD pipe (WPIP) via dadDoConnect'
	read(5,*) iopt
	if (iopt.lt.0.or.iopt.gt.6) stop 'Invalid selection'
901	format(8x,i1,':',4x,a)

	write(6,'(/a)') 'Do you wish to use key tables? [<CR>=y]'
	read(5,'(a)') cline
	use_key = .true.
	if (cline(1:1).eq.'n' .or. cline(1:1).eq.'N') use_key = .false.

! The input filenames we choose follow certain conventions

! ... IE format carries .ie as extension
	if (iopt.eq.0) then
	  cfile = 'toyMC.ie'

! ... FZ files usually carry .events extensions
	else if (iopt.eq.1) then
	  cfile = 'toyMC.events'

! ... DAD files are often called .devents (or .dfil)
	else if (iopt.le.5) then
	  cfile = 'toyMC.devents'

! ... pipes are often called .pipe
	else 
	  cfile = 'toyMC.pipe'
	endif

	write(6,'(2a)') ' ---> Reading from ', cfile

!=====================================================================
!  INITIALIZE ADAMO
!=====================================================================

! The INITAP routine created by MAD initializes ADAMO and also calls 
! the CREOBJ routine to set up your personal data structures.

	call INITAP

!=====================================================================
!  OPEN AN EXISTING GAF FILE
!=====================================================================

! Set up the appropriate DAD flags for reading

	if (use_key) then
	  ckey = 'mcEvKey'
	  dadflags = CAC_READ + CAC_KEYTABLE
	else
	  ckey = ' '
	  dadflags = CAC_READ
	endif

! OPTIONS 0-1: Open a GAF in IE or FZ format using OPEGAF
! -----------

	if (iopt.le.1) then
	  if (iopt.eq.0) then
	    cdriver = 'IE'
	  else 
	    cdriver = 'FZ,FILFOR=EXCH'
	  endif
	  cline = 'NAME='//cfile(1:lnblnk(cfile))//',DRIVER='//cdriver
	  call OPEGAF (igaf,cline,iok)

! OPTION 2: Open a DAD file for reading via dadOpenFile
! --------

	else if (iopt.eq.2) then
	  call dadOpenFile (
     +		igaf,			! identifier for the new file
     +		cfile,			! file name
     +		'mcEvents',		! dataflow name
     +		ckey,			! name of key table
     +		dadflags,		! sum of flags
     +		iok )			! error code

! OPTIONS 3-6: These more generic techniques can be used to read from files,
! -----------  pipes, and servers, all termed 'hosts'. In all cases, the host 
!              must first be defined, by providing this information:
!		   - NAME of host or file
!		   - TYPE of host: file (WFIL), pipe (WPIP), or server (INET)
!		   - PORT number (for server connections only)
!		   - DATAFLOW name to work with
!		   - FLAGS (see variable dadflags above)
!              Then, you simply 'connect' to the chosen host.

! OPTION 3: Open a DAD file for reading using dadDoConnect.
! --------  The dadDoConnect routine provides a ONE-SHOT way 
!	    to simultaneously define the host and connect to it. 

	else if (iopt.eq.3) then

	  call dadDoConnect (
     +		igaf,			! identifier for the new file
     +		cfile,			! file name
     +		'RFIL',			! connection type
     +		0,                      ! port (only needed for servers)
     +		'mcEvents',		! dataflow name
     +		ckey,			! name of key table
     +		dadflags,		! sum of flags
     +		iok )			! error code

! OPTION 4: Open a DAD file for reading using dadHost and dadConnect.
! --------  The host definition is performed here by a call to dadHost.

	else if (iopt.eq.4) then

! ... first, define the host
	  call dadHost (
     +		cfile,		        ! host or file name
     +		'RFIL',			! connection type
     +		0,			! port number, 0 for local connection
     +		'mcEvents',             ! dataflow name
     +		dadflags )              ! sum of flags

! ... then open the connection
	  call dadConnect (
     +		'mcEvents',		! dataflow name
     +		igaf,			! identifier for the new file
     +		iok )			! error code

! OPTION 5: Open a DAD file for reading using dadInit and dadConnect.
! --------  The host definitions here are supplied by an EXTERNAL FILE, 
!	    named dadinit.cnf, which you must provide. For this program 
!	    to work, your dadinit.cnf file must contain the line:
!		        toyMC.devents RFIL 0 mcEvents rk
!	    The information is the same as that in the dadHost call above.
!	    'rk' is shorthand for the dadflags codes (see the dad manual).

	else if (iopt.eq.5) then

! ... read in the host definitions from dadinit.cnf
	  call dadInit

! ... then open the connection
	  call dadConnect (
     +		'mcEvents',             ! dataflow name
     +		igaf,                   ! identifier for the new file
     +		iok )                   ! error code

! OPTION 6: Open a DAD pipe for reading using dadDoConnect.
! --------  Practically identical to OPTION 3. You can also use dadOpenPipe, 
!	    which allows for split pipes! (see the dad manual)

	else if (iopt.eq.6) then

	  call dadDoConnect (
     +		igaf,			! identifier for the new file
     +		cfile,			! file name
     +		'RPIP',			! connection type
     +		0,                      ! port (only needed for servers)
     +		'mcEvents',		! dataflow name
     +		ckey,			! name of key table
     +		dadflags,		! sum of flags
     +		iok )			! error code

	endif

! Check that the input file was opened successfully.

	if (iok.ne.0) stop 'Failed to open input file.'

!=====================================================================
!      **********    START LOOP OVER EVENTS    **********
!=====================================================================

10	continue

!=====================================================================
!   READ IN THE NEXT RECORD
!=====================================================================

! OPTIONS 0-1: Retrieve the key table information for the next record 
! -----------  using NEXGAF. Next, check for EOF. Finally, actually
!	       load the record into ADAMO memory using FETGAF.

	if (iopt.le.1) then

! ... pull key table info for the next record into the WCB
	  call NEXGAF (igaf, mcEvKey, iok)

! ... leave the loop if at end of file (EOF return code is -1)
	  if (iok.ne.0) goto 999

! ... if this is the first event, obtain dataflow name from entry
! ... cName of key table, to prove this is working
	  if (mcEvKey_iEvent.eq.1) then
	    write(6,905) 'mcEvKey_cName = ', mcEvKey_cName, 
     +		' (should be the dataflow name, mcEvents)'
	    idfl = GETTDF (mcEvKey_cName)
	  endif

! ... call FETGAF to read the tables from this record into memory.
! ... Note: 'this record' = that currently stored in the key table WIN.
! ... Since we just loaded this WIN with NEXGAF, we will be getting the 
! ... NEXT record, as planned.
	  call FETGAF (igaf, mcEvKey, idfl, iok)

! OPTIONS 2-6: Use dadRead to retrieve the next record. It automatically
! -----------  stores all tables into ADAMO memory. (All except the key
!	       table, which is anyway not a 'real' table).
!	       
	else

! ... load tables from next record into memory using dadRead
	  call dadRead (igaf, iok)

! ... leave the loop if at end of file 
	  if (iok.ne.0) goto 999

! ... The key table info for this record has been stored in a dad-internal
! ... area. If we want to use it, we have to explicitly copy this info
! ... into our local Window Common Block using dadGetRecordKey
	  if (use_key) call dadGetRecordKey (igaf, mcEvKey, iok)

! ... if this is the first event, obtain dataflow name from entry
! ... cName of key table, to prove this is working
	  if (mcEvKey_iEvent.eq.1) then
	    write(6,905) 'mcEvKey_cName = ', mcEvKey_cName, 
     +		' (should be the dataflow name, mcEvents)'
	    idfl = GETTDF (mcEvKey_cName)
	  endif

	endif

905	format(3x,a,a10,a)

!=====================================================================
!  RETRIEVE AND PRINT SOME INFORMATION FROM TABLES
!=====================================================================

! Retrieve some info from mcEvent and mcTrack

	call FETTAB (mcEvent,ID,1)
	ntracks = COUTAB(mcTrack)
	if (ntracks .gt. maxtracks) then
	  print *, 'ERROR: more than ', maxtracks, ' tracks in mcTrack'
	  ntracks = maxtracks
	endif
	do i = 1, ntracks
	  call FETTAB (mcTrack,ID,i)
	  momentum(i) = mcTrack_p
	enddo

! And print it out, to make sure all is working
! Note: the event number comes from the key table

	write(6,800) 'EVENT ', mcEvKey_ievent, ': Q2 = ', mcEvent_Q2, 
     +		', ', coutab(mcTrack),
     +		' tracks with p = ', (momentum(i),i=1,ntracks)
800	format(a,i2,a,f5.2,a,i2,a,<maxtracks>(f4.1,4x))

!=====================================================================
!      **********    END LOOP OVER RECORDS   **********
!=====================================================================

! Go back for the next record

	goto 10

! We end up here when EOF is reached

999	continue

!=====================================================================
!    CLOSE THE GAF
!=====================================================================

! OPTIONS 0-1: Close the GAF using CLOGAF
! -----------

	if (iopt .le. 1) then
	  call CLOGAF (igaf,iok)

! OPTION 2: Close the DAD file using dadCloseFile (note, you can also
! --------  use dadDisConnect ... dadCloseFile is just a special case)

	else if (iopt .eq. 2) then
	  call dadCloseFile (igaf, iok)

! OPTIONS 3-6: Close the generic DAD host using dadDisConnect
! -----------

	else
	  call dadDisConnect (igaf,iok)
	endif

	print *

	end

