
	program write

!=====================================================================
!  DECLARATIONS
!=====================================================================

	implicit none

! Include files needed for ADAMO and DAD use

#include "partap.inc"
#include "dad/command.inc"

! Window common block (WCB) definitions for all tables you want to use

#include "mcEvKey.inc"
#include "mcEvent.inc"
#include "mcTrack.inc"

! Finally declare some variables of your own

	real		ebeam, q2min, q2max, nevents
	parameter	(ebeam = 27.5)
	parameter	(q2min = 1.0)
	parameter	(q2max = 20.0)
	parameter	(nevents = 10)
	logical		use_key
	integer		i, iopt, igaf, idfl, iok, ievent, dadflags
	character	ckey*80, cline*80, cfile*80, cdriver*4

! ... external functions are declared just like variables
	real		notrandom
	integer		lnblnk		! useful CERNLIB routine

!=====================================================================
!  ASK USER TO SELECT OPTION: TYPE OF FILE TO WRITE
!=====================================================================

	write(6,'(/a)') 'Please select type of file to write: [<CR>=0]'
	write(6,901) 0, 'GAF in IE format'
	write(6,901) 1, 'GAF in FZ format'
	write(6,901) 2, 'DAD file (WFIL) via dadOpenFile'
	write(6,901) 3, 'DAD file (WFIL) via dadDoConnect'
	write(6,901) 4, 'DAD file (WFIL) via dadHost, dadConnect'
	write(6,901) 5, 'DAD file (WFIL) via dadInit, dadConnect'
	write(6,901) 6, 'DAD pipe (WPIP) via dadDoConnect'
	read(5,*) iopt
	if (iopt.lt.0.or.iopt.gt.6) stop 'Invalid selection'
901	format(8x,i1,':',4x,a)

	write(6,'(/a)') 'Do you wish to use key tables? [<CR>=y]'
	read(5,'(a)') cline
	use_key = .true.
	if (cline(1:1).eq.'n' .or. cline(1:1).eq.'N') use_key = .false.

! The output filenames we choose follow certain conventions

! ... IE format carries .ie as extension
	if (iopt.eq.0) then
	  cfile = 'toyMC.ie'

! ... FZ files usually carry .events extensions
	else if (iopt.eq.1) then
	  cfile = 'toyMC.events'

! ... DAD files are often called .devents (or .dfil)
	else if (iopt.le.5) then
	  cfile = 'toyMC.devents'

! ... pipes are often called .pipe
	else 
	  cfile = 'toyMC.pipe'
	  write(6,'(2a)') ' ---> Writing to pipe ', cfile
	endif


!=====================================================================
!  INITIALIZE ADAMO
!=====================================================================

! The INITAP routine created by MAD initializes ADAMO and also calls 
! the CREOBJ routine to set up your personal data structures.

	call INITAP

!=====================================================================
!  OPEN A NEW GAF FILE
!=====================================================================

! Set up the appropriate DAD flags for writing

	if (use_key) then
	  ckey = 'mcEvKey'
	  dadflags = CAC_WRITE + CAC_KEYTABLE
! ........ NOTE: the key table field MUST be left blank here!
	else
	  ckey = ' '
	  dadflags = CAC_WRITE
	endif

! OPTIONS 0-1: Create a new GAF in IE or FZ format using CREGAF
! -----------

	if (iopt.le.1) then
	  if (iopt.eq.0) then
	    cdriver = 'IE'
	  else 
	    cdriver = 'FZ,FILFOR=EXCH'
	  endif
	  cline = 'NAME='//cfile(1:lnblnk(cfile))//',DRIVER='//cdriver
	  call CREGAF (igaf,mcEvKey,cline,iok)

! OPTION 2: Open a DAD file for writing via dadOpenFile
! --------

	else if (iopt.eq.2) then
	  call dadOpenFile (
     +		igaf,			! identifier for the new file
     +		cfile,			! file name
     +		'mcEvents',		! dataflow name
     +		ckey,			! name of key table
     +		dadflags,		! sum of flags
     +		iok )			! error code

! OPTIONS 3-6: These more generic techniques can be used to write to files,
! -----------  pipes, and servers, all termed 'hosts'. In all cases, the host 
!              must first be defined, by providing this information:
!		   - NAME of host or file
!		   - TYPE of host: file (WFIL), pipe (WPIP), or server (INET)
!		   - PORT number (for server connections only)
!		   - DATAFLOW name to work with
!		   - FLAGS (see variable dadflags above)
!              Then, you simply 'connect' to the chosen host.

! OPTION 3: Open a DAD file for writing using dadDoConnect.
! --------  The dadDoConnect routine provides a ONE-SHOT way 
!	    to simultaneously define the host and connect to it. 

	else if (iopt.eq.3) then

	  call dadDoConnect (
     +		igaf,			! identifier for the new file
     +		cfile,			! file name
     +		'WFIL',			! connection type
     +		0,                      ! port (only needed for servers)
     +		'mcEvents',		! dataflow name
     +		ckey,			! name of key table
     +		dadflags,		! sum of flags
     +		iok )			! error code

! OPTION 4: Open a DAD file for writing using dadHost and dadConnect.
! --------  The host definition is performed here by a call to dadHost.
!	    Note that I cannot get keytable writing to work here ...

	else if (iopt.eq.4) then

! ... first, define the host
	  call dadHost (
     +		cfile,			! host or file name
     +		'WFIL',			! connection type
     +		0,			! port number, 0 for local connection
     +		'mcEvents',             ! dataflow name
     +		dadflags )              ! sum of flags

! ... then open the connection
	  call dadConnect (
     +		'mcEvents',		! dataflow name
     +		igaf,			! identifier for the new file
     +		iok )			! error code

! OPTION 5: Open a DAD file for writing using dadInit and dadConnect.
! --------  The host definitions here are supplied by an EXTERNAL FILE, 
!	    named dadinit.cnf, which you must provide. For this program
!	    to work, your dadinit.cnf file must contain the line:
!		        toyMC.devents WFIL 0 mcEvents wk
!	    The information is the same as that in the dadHost call.
!	    'wk' is shorthand for the dadflags codes (see the dad manual).
!	    Note that I cannot get keytable writing to work here ...

	else if (iopt.eq.5) then

! ... read in the host definitions from dadinit.cnf
	  call dadInit

! ... then open the connection
	  call dadConnect (
     +		'mcEvents',             ! dataflow name
     +		igaf,                   ! identifier for the new file
     +		iok )                   ! error code

! OPTION 6: Open a DAD PIPE for writing using dadDoConnect.
! --------  Practically identical to OPTION 3. You can also use dadOpenPipe, 
!	    which allows for split pipes! (see the dad manual)

	else if (iopt.eq.6) then

	  call dadDoConnect (
     +		igaf,			! identifier for the new file
     +		cfile,			! file name
     +		'WPIP',			! connection type
     +		0,                      ! port (only needed for servers)
     +		'mcEvents',		! dataflow name
     +		ckey,			! name of key table
     +		dadflags,		! sum of flags
     +		iok )			! error code

	endif

! Check that the output file was opened successfully.

	if (iok.ne.0) stop 'Failed to open output file.'

!=====================================================================
!      **********    START LOOP OVER EVENTS    **********
!=====================================================================

	do ievent = 1, nevents

	  write(6,'(20x,a,i3,1x,a)') '<<<<< WRITING EVENT NUMBER ', 
     +		ievent, '>>>>>'      

!=====================================================================
!  GENERATE THE EVENT
!=====================================================================

! Clear tables.
! NOTE: when applied to a dataflow, CLETAB only clears the tables that are 
! located IMMEDIATELY inside it ... CLETAB will NOT navigate down a tree 
! of dataflows.

	  call CLETAB (GETTDF('mcEvents'))

! Fill the mcEvent table

	  mcEvent_ID = 1
	  mcEvent_Nu = notrandom()*ebeam
	  mcEvent_Q2 = q2min + (q2max-q2min)*notrandom()
	  call INSTAB (mcEvent)

! Fill mcTrack with 4 random particle tracks

	  do i = 1, 4
	    mcTrack_ID = NEXT
	    mcTrack_P = mcEvent_Nu * notrandom()
	    call INSTAB (mcTrack)
	  enddo

!=====================================================================
!  STORE THE EVENT RECORD TO THE GAF
!=====================================================================

! Fill the key table WCB to identify this record (record = 'event' here)
! Note: There is no need to load the key table to memory with INSTAB.

	  mcEvKey_ID = 1
	  mcEvKey_iRun = 1
	  mcEvKey_iEvent = ievent
	  mcEvKey_cName = 'mcEvents'

! ... DAD only: we must explicitly copy our WCB to the dad-internal version
	  if (iopt.gt.1 .and. use_key)  then
	    call dadFillRecordKey (igaf, mcEvKey, iok)
	  endif
	  
! OPTIONS 0-1: Store the record in the GAF using INSGAF
! -----------

	  if (iopt.le.1) then
	    call INSGAF (igaf,mcEvKey,iok)

! OPTIONS 2-6: Store the record in the DAD file using dadWrite
! -----------

	  else
	    call dadWrite (igaf, iok)

	  endif

!=====================================================================
!      **********    END LOOP OVER EVENTS   **********
!=====================================================================

	enddo ! <loop over events>

!=====================================================================
!  CLOSE THE GAF
!=====================================================================

! OPTIONS 0-1: Close the GAF using CLOGAF
! -----------

	if (iopt .le. 1) then
	  call CLOGAF (igaf,iok)

! OPTION 2: Close the DAD file using dadCloseFile (note, you can also
! --------  use dadDisConnect ... dadCloseFile is just a special case)

	else if (iopt .eq. 2) then
	  call dadCloseFile (igaf, iok)

! OPTIONS 3-6: Close the generic DAD host using dadDisConnect
! -----------

	else
	  call dadDisConnect (igaf,iok)
	endif

! Tell the user what we did

	write(6,'(/2a/)') ' ---> Wrote file: ', cfile

	end

!=====================================================================
!  CODING EXAMPLE: the NOTRANDOM number generator :)
!=====================================================================

	real function notrandom()

	real		increment
	parameter	(increment=0.09)
	real		current_number
	data		current_number/0/
	save		current_number

	current_number = current_number + increment
	current_number = mod(current_number,1)
	notrandom = current_number
	
	return

	end

